package kr.co.vividnext.sodalive.content.comment

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.comment.QAudioContentComment.audioContentComment
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository
import java.time.ZoneId
import java.time.format.DateTimeFormatter

@Repository
interface AudioContentCommentRepository : JpaRepository<AudioContentComment, Long>, AudioContentCommentQueryRepository

interface AudioContentCommentQueryRepository {
    fun findByContentId(
        cloudFrontHost: String,
        contentId: Long,
        timezone: String,
        offset: Long,
        limit: Int
    ): List<GetAudioContentCommentListItem>

    fun totalCountCommentByContentId(contentId: Long): Int
    fun commentReplyCountByAudioContentCommentId(commentId: Long): Int
    fun getAudioContentCommentReplyList(
        cloudFrontHost: String,
        commentId: Long,
        timezone: String,
        offset: Long,
        limit: Int
    ): List<GetAudioContentCommentListItem>

    fun findPushTokenByContentIdAndCommentParentIdMyMemberId(
        contentId: Long,
        commentParentId: Long?,
        myMemberId: Long
    ): List<FindPushTokenByContentIdAndCommentParentIdMyMemberIdResponse>
}

@Repository
class AudioContentCommentQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : AudioContentCommentQueryRepository {
    override fun findByContentId(
        cloudFrontHost: String,
        contentId: Long,
        timezone: String,
        offset: Long,
        limit: Int
    ): List<GetAudioContentCommentListItem> {
        return queryFactory.selectFrom(audioContentComment)
            .where(
                audioContentComment.audioContent.id.eq(contentId)
                    .and(audioContentComment.isActive.isTrue)
                    .and(audioContentComment.parent.isNull)
            )
            .offset(offset)
            .limit(limit.toLong())
            .orderBy(audioContentComment.createdAt.desc())
            .fetch()
            .asSequence()
            .map {
                val date = it.createdAt!!
                    .atZone(ZoneId.of("UTC"))
                    .withZoneSameInstant(ZoneId.of(timezone))

                GetAudioContentCommentListItem(
                    id = it.id!!,
                    writerId = it.member!!.id!!,
                    nickname = it.member!!.nickname,
                    profileUrl = if (it.member!!.profileImage != null) {
                        "$cloudFrontHost/${it.member!!.profileImage}"
                    } else {
                        "$cloudFrontHost/profile/default-profile.png"
                    },
                    comment = it.comment,
                    donationCan = it.donationCan ?: 0,
                    date = date.format(DateTimeFormatter.ofPattern("yyyy.MM.dd E hh:mm a")),
                    replyCount = commentReplyCountByAudioContentCommentId(it.id!!)
                )
            }
            .toList()
    }

    override fun totalCountCommentByContentId(contentId: Long): Int {
        return queryFactory.select(audioContentComment.id)
            .from(audioContentComment)
            .where(
                audioContentComment.audioContent.id.eq(contentId)
                    .and(audioContentComment.parent.isNull)
                    .and(audioContentComment.isActive.isTrue)
            )
            .fetch()
            .size
    }

    override fun commentReplyCountByAudioContentCommentId(commentId: Long): Int {
        return queryFactory.select(audioContentComment.id)
            .from(audioContentComment)
            .where(
                audioContentComment.parent.isNotNull
                    .and(audioContentComment.parent.id.eq(commentId))
                    .and(audioContentComment.isActive.isTrue)
            )
            .fetch()
            .size
    }

    override fun getAudioContentCommentReplyList(
        cloudFrontHost: String,
        commentId: Long,
        timezone: String,
        offset: Long,
        limit: Int
    ): List<GetAudioContentCommentListItem> {
        return queryFactory.selectFrom(audioContentComment)
            .where(
                audioContentComment.parent.isNotNull
                    .and(audioContentComment.parent.id.eq(commentId))
                    .and(audioContentComment.isActive.isTrue)
            )
            .offset(offset)
            .limit(limit.toLong())
            .orderBy(audioContentComment.createdAt.desc())
            .fetch()
            .asSequence()
            .map {
                val date = it.createdAt!!
                    .atZone(ZoneId.of("UTC"))
                    .withZoneSameInstant(ZoneId.of(timezone))

                GetAudioContentCommentListItem(
                    id = it.id!!,
                    writerId = it.member!!.id!!,
                    nickname = it.member!!.nickname,
                    profileUrl = if (it.member!!.profileImage != null) {
                        "$cloudFrontHost/${it.member!!.profileImage}"
                    } else {
                        "$cloudFrontHost/profile/default-profile.png"
                    },
                    comment = it.comment,
                    donationCan = it.donationCan ?: 0,
                    date = date.format(DateTimeFormatter.ofPattern("yyyy.MM.dd E hh:mm a")),
                    replyCount = 0
                )
            }
            .toList()
    }

    override fun findPushTokenByContentIdAndCommentParentIdMyMemberId(
        contentId: Long,
        commentParentId: Long?,
        myMemberId: Long
    ): List<FindPushTokenByContentIdAndCommentParentIdMyMemberIdResponse> {
        var where = audioContent.id.eq(contentId)
            .and(member.id.ne(myMemberId))
            .and(audioContentComment.isActive.isTrue)

        if (commentParentId != null) {
            where = where.and(
                audioContentComment.parent.id.eq(commentParentId)
                    .or(audioContentComment.id.eq(commentParentId))
            )
        }

        val response = if (commentParentId != null) {
            queryFactory
                .select(
                    QFindPushTokenByContentIdAndCommentParentIdMyMemberIdResponse(
                        member.pushToken.coalesce(""),
                        member.container
                    )
                )
                .from(audioContentComment)
                .innerJoin(audioContentComment.audioContent, audioContent)
                .innerJoin(audioContentComment.member, member)
                .where(where)
                .fetch()
        } else {
            queryFactory
                .select(
                    QFindPushTokenByContentIdAndCommentParentIdMyMemberIdResponse(
                        member.pushToken.coalesce(""),
                        member.container
                    )
                )
                .from(audioContent)
                .innerJoin(audioContent.member, member)
                .where(where)
                .fetch()
        }

        return response
    }
}
