package kr.co.vividnext.sodalive.explorer.profile.creatorCommunity

import com.amazonaws.services.s3.model.ObjectMetadata
import com.fasterxml.jackson.databind.ObjectMapper
import kr.co.vividnext.sodalive.aws.s3.S3Uploader
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.comment.CreatorCommunityComment
import kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.comment.CreatorCommunityCommentRepository
import kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.comment.GetCommunityPostCommentListResponse
import kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.comment.ModifyCommunityPostCommentRequest
import kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.like.CreatorCommunityLike
import kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.like.CreatorCommunityLikeRepository
import kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.like.PostCommunityPostLikeRequest
import kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.like.PostCommunityPostLikeResponse
import kr.co.vividnext.sodalive.fcm.FcmEvent
import kr.co.vividnext.sodalive.fcm.FcmEventType
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.block.BlockMemberRepository
import kr.co.vividnext.sodalive.utils.generateFileName
import org.springframework.beans.factory.annotation.Value
import org.springframework.context.ApplicationEventPublisher
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import org.springframework.web.multipart.MultipartFile
import java.time.Duration
import java.time.LocalDateTime

@Service
class CreatorCommunityService(
    private val repository: CreatorCommunityRepository,
    private val blockMemberRepository: BlockMemberRepository,
    private val likeRepository: CreatorCommunityLikeRepository,
    private val commentRepository: CreatorCommunityCommentRepository,

    private val s3Uploader: S3Uploader,
    private val objectMapper: ObjectMapper,
    private val applicationEventPublisher: ApplicationEventPublisher,

    @Value("\${cloud.aws.s3.bucket}")
    private val imageBucket: String,

    @Value("\${cloud.aws.cloud-front.host}")
    private val imageHost: String
) {
    @Transactional
    fun createCommunityPost(postImage: MultipartFile?, requestString: String, member: Member) {
        val request = objectMapper.readValue(requestString, CreateCommunityPostRequest::class.java)

        val post = CreatorCommunity(
            content = request.content,
            isCommentAvailable = request.isCommentAvailable,
            isAdult = request.isAdult
        )
        post.member = member
        repository.save(post)

        if (postImage != null) {
            val metadata = ObjectMetadata()
            metadata.contentLength = postImage.size

            // 커버 이미지 파일명 생성
            val imageFileName = generateFileName(prefix = "${post.id}-image")

            // 커버 이미지 업로드
            val imagePath = s3Uploader.upload(
                inputStream = postImage.inputStream,
                bucket = imageBucket,
                filePath = "creator_community/${post.id}/$imageFileName",
                metadata = metadata
            )

            post.imagePath = imagePath
        }

        applicationEventPublisher.publishEvent(
            FcmEvent(
                type = FcmEventType.CHANGE_NOTICE,
                title = member.nickname,
                message = "새 글이 등록되었습니다.",
                creatorId = member.id!!
            )
        )
    }

    @Transactional
    fun modifyCommunityPost(postImage: MultipartFile?, requestString: String, member: Member) {
        val request = objectMapper.readValue(requestString, ModifyCommunityPostRequest::class.java)

        val post = repository.findByIdAndMemberId(id = request.creatorCommunityId, memberId = member.id!!)
            ?: throw SodaException("잘못된 요청입니다.")

        if (request.content != null) {
            post.content = request.content
        }

        if (request.isCommentAvailable != null) {
            post.isCommentAvailable = request.isCommentAvailable
        }

        if (request.isAdult != null) {
            post.isAdult = request.isAdult
        }

        if (request.isActive != null) {
            post.isActive = request.isActive
        }

        if (postImage != null) {
            val metadata = ObjectMetadata()
            metadata.contentLength = postImage.size

            // 커버 이미지 파일명 생성
            val imageFileName = generateFileName(prefix = "${post.id}-image")

            // 커버 이미지 업로드
            val imagePath = s3Uploader.upload(
                inputStream = postImage.inputStream,
                bucket = imageBucket,
                filePath = "creator_community/${post.id}/$imageFileName",
                metadata = metadata
            )

            post.imagePath = imagePath
        }
    }

    fun getCommunityPostList(
        creatorId: Long,
        memberId: Long,
        timezone: String,
        offset: Long,
        limit: Long,
        isAdult: Boolean
    ): List<GetCommunityPostListResponse> {
        val postList = repository.getCommunityPostList(
            creatorId = creatorId,
            offset = offset,
            limit = limit,
            isAdult = isAdult
        )

        return postList
            .asSequence()
            .filter {
                !blockMemberRepository.isBlocked(
                    blockedMemberId = memberId,
                    memberId = creatorId
                )
            }
            .map {
                val isLike =
                    likeRepository.findByPostIdAndMemberId(postId = it.id!!, memberId = memberId)?.isActive ?: false
                val likeCount = likeRepository.totalCountCommunityPostLikeByPostId(it.id!!)
                val commentCount = if (it.isCommentAvailable) {
                    commentRepository.totalCountCommentByPostId(postId = it.id!!)
                } else {
                    0
                }
                val commentList = if (it.isCommentAvailable) {
                    commentRepository.findByPostId(
                        cloudFrontHost = imageHost,
                        timezone = timezone,
                        id = it.id!!,
                        offset = 0,
                        limit = 1
                    )
                } else {
                    listOf()
                }

                val firstComment = if (it.isCommentAvailable && commentList.isNotEmpty()) {
                    commentList[0]
                } else {
                    null
                }

                GetCommunityPostListResponse(
                    postId = it.id!!,
                    creatorId = it.member!!.id!!,
                    creatorNickname = it.member!!.nickname,
                    creatorProfileUrl = if (it.member!!.profileImage != null) {
                        "$imageHost/${it.member!!.profileImage}"
                    } else {
                        "$imageHost/profile/default-profile.png"
                    },
                    imageUrl = if (it.imagePath != null) {
                        "$imageHost/${it.imagePath!!}"
                    } else {
                        null
                    },
                    content = it.content,
                    date = getTimeAgoString(it.createdAt!!),
                    isCommentAvailable = it.isCommentAvailable,
                    isAdult = it.isAdult,
                    isLike = isLike,
                    likeCount = likeCount,
                    commentCount = commentCount,
                    firstComment = firstComment
                )
            }
            .toList()
    }

    fun getCommunityPostDetail(
        postId: Long,
        memberId: Long,
        timezone: String,
        isAdult: Boolean
    ): GetCommunityPostListResponse {
        val post = repository.findByIdAndActive(postId, isAdult = isAdult)
            ?: throw SodaException("잘못된 요청입니다.\n다시 시도해 주세요.")

        val isBlocked = blockMemberRepository.isBlocked(blockedMemberId = memberId, memberId = post.member!!.id!!)
        if (isBlocked) throw SodaException("${post.member!!.nickname}님의 요청으로 접근이 제한됩니다.")

        val isLike = likeRepository.findByPostIdAndMemberId(postId = post.id!!, memberId = memberId)?.isActive ?: false
        val likeCount = likeRepository.totalCountCommunityPostLikeByPostId(post.id!!)
        val commentCount = if (post.isCommentAvailable) {
            commentRepository.totalCountCommentByPostId(postId = post.id!!)
        } else {
            0
        }
        val commentList = if (post.isCommentAvailable) {
            commentRepository.findByPostId(
                cloudFrontHost = imageHost,
                timezone = timezone,
                id = post.id!!,
                offset = 0,
                limit = 1
            )
        } else {
            listOf()
        }

        val firstComment = if (post.isCommentAvailable && commentList.isNotEmpty()) {
            commentList[0]
        } else {
            null
        }

        return GetCommunityPostListResponse(
            postId = post.id!!,
            creatorId = post.member!!.id!!,
            creatorNickname = post.member!!.nickname,
            creatorProfileUrl = if (post.member!!.profileImage != null) {
                "$imageHost/${post.member!!.profileImage}"
            } else {
                "$imageHost/profile/default-profile.png"
            },
            imageUrl = if (post.imagePath != null) {
                "$imageHost/${post.imagePath!!}"
            } else {
                null
            },
            content = post.content,
            date = getTimeAgoString(post.createdAt!!),
            isCommentAvailable = post.isCommentAvailable,
            isAdult = post.isAdult,
            isLike = isLike,
            likeCount = likeCount,
            commentCount = commentCount,
            firstComment = firstComment
        )
    }

    private fun getTimeAgoString(dateTime: LocalDateTime): String {
        val now = LocalDateTime.now()
        val duration = Duration.between(dateTime, now)

        return when {
            duration.toMinutes() < 60 -> "${duration.toMinutes()}분전"
            duration.toHours() < 24 -> "${duration.toHours()}시간전"
            duration.toDays() < 30 -> "${duration.toDays()}일전"
            duration.toDays() < 365 -> "${duration.toDays() / 30}개월전"
            else -> "${duration.toDays() / 365}년전"
        }
    }

    @Transactional
    fun communityPostLike(request: PostCommunityPostLikeRequest, member: Member): PostCommunityPostLikeResponse {
        var postLike = likeRepository.findByPostIdAndMemberId(postId = request.postId, memberId = member.id!!)

        if (postLike == null) {
            postLike = CreatorCommunityLike()
            postLike.member = member

            val post = repository.findByIdAndActive(request.postId, isAdult = member.auth != null)
                ?: throw SodaException("잘못된 요청입니다.\n다시 시도해 주세요.")

            postLike.creatorCommunity = post
            likeRepository.save(postLike)
        } else {
            postLike.isActive = !postLike.isActive
        }

        return PostCommunityPostLikeResponse(like = postLike.isActive)
    }

    @Transactional
    fun createCommunityPostComment(comment: String, postId: Long, parentId: Long?, member: Member) {
        val post = repository.findByIdOrNull(id = postId)
            ?: throw SodaException("잘못된 게시물 입니다.\n다시 시도해 주세요.")

        val postComment = CreatorCommunityComment(comment = comment)
        postComment.creatorCommunity = post
        postComment.member = member

        val parent = if (parentId != null) {
            commentRepository.findByIdOrNull(id = parentId)
        } else {
            null
        }

        if (parent != null) {
            postComment.parent = parent
        }

        commentRepository.save(postComment)
    }

    @Transactional
    fun modifyCommunityPostComment(request: ModifyCommunityPostCommentRequest, member: Member) {
        val postComment = commentRepository.findByIdOrNull(id = request.commentId)
            ?: throw SodaException("잘못된 접근 입니다.\n확인 후 다시 시도해 주세요.")

        if (request.comment != null && postComment.member!!.id!! == member.id!!) {
            postComment.comment = request.comment
        }

        if (
            request.isActive != null &&
            (
                postComment.member!!.id!! == member.id!! ||
                    postComment.creatorCommunity!!.member!!.id!! == member.id!!
                )
        ) {
            postComment.isActive = request.isActive
        }
    }

    fun getCommunityPostCommentList(
        postId: Long,
        timezone: String,
        offset: Long,
        limit: Long
    ): GetCommunityPostCommentListResponse {
        val commentList = commentRepository.findByPostId(
            cloudFrontHost = imageHost,
            timezone = timezone,
            id = postId,
            offset = offset,
            limit = limit
        )

        val totalCount = commentRepository.totalCountCommentByPostId(postId = postId)
        return GetCommunityPostCommentListResponse(totalCount = totalCount, items = commentList)
    }

    fun getCommentReplyList(
        commentId: Long,
        timezone: String,
        offset: Long,
        limit: Long
    ): GetCommunityPostCommentListResponse {
        val commentList = commentRepository.getCommunityCommentReplyList(
            cloudFrontHost = imageHost,
            commentId = commentId,
            timezone = timezone,
            offset = offset,
            limit = limit
        )

        val totalCount = commentRepository.commentReplyCountByCommentId(commentId)
        return GetCommunityPostCommentListResponse(totalCount = totalCount, items = commentList)
    }

    fun getLatestPostListFromCreatorsYouFollow(
        timezone: String,
        memberId: Long,
        isAdult: Boolean
    ): List<GetCommunityPostListResponse> {
        val postList = repository.getLatestPostListFromCreatorsYouFollow(memberId = memberId, isAdult = isAdult)

        return postList
            .asSequence()
            .filter {
                !blockMemberRepository.isBlocked(
                    blockedMemberId = memberId,
                    memberId = it.member!!.id!!
                )
            }
            .map {
                val isLike =
                    likeRepository.findByPostIdAndMemberId(postId = it.id!!, memberId = memberId)?.isActive ?: false
                val likeCount = likeRepository.totalCountCommunityPostLikeByPostId(it.id!!)
                val commentCount = if (it.isCommentAvailable) {
                    commentRepository.totalCountCommentByPostId(postId = it.id!!)
                } else {
                    0
                }
                val commentList = if (it.isCommentAvailable) {
                    commentRepository.findByPostId(
                        cloudFrontHost = imageHost,
                        timezone = timezone,
                        id = it.id!!,
                        offset = 0,
                        limit = 1
                    )
                } else {
                    listOf()
                }

                val firstComment = if (it.isCommentAvailable && commentList.isNotEmpty()) {
                    commentList[0]
                } else {
                    null
                }

                GetCommunityPostListResponse(
                    postId = it.id!!,
                    creatorId = it.member!!.id!!,
                    creatorNickname = it.member!!.nickname,
                    creatorProfileUrl = if (it.member!!.profileImage != null) {
                        "$imageHost/${it.member!!.profileImage}"
                    } else {
                        "$imageHost/profile/default-profile.png"
                    },
                    imageUrl = if (it.imagePath != null) {
                        "$imageHost/${it.imagePath!!}"
                    } else {
                        null
                    },
                    content = it.content,
                    date = getTimeAgoString(it.createdAt!!),
                    isCommentAvailable = it.isCommentAvailable,
                    isAdult = it.isAdult,
                    isLike = isLike,
                    likeCount = likeCount,
                    commentCount = commentCount,
                    firstComment = firstComment
                )
            }
            .toList()
    }
}
