package kr.co.vividnext.sodalive.live.roulette

import kr.co.vividnext.sodalive.can.CanRepository
import kr.co.vividnext.sodalive.can.charge.Charge
import kr.co.vividnext.sodalive.can.charge.ChargeRepository
import kr.co.vividnext.sodalive.can.charge.ChargeStatus
import kr.co.vividnext.sodalive.can.payment.CanPaymentService
import kr.co.vividnext.sodalive.can.payment.Payment
import kr.co.vividnext.sodalive.can.payment.PaymentGateway
import kr.co.vividnext.sodalive.can.payment.PaymentStatus
import kr.co.vividnext.sodalive.can.use.CanUsage
import kr.co.vividnext.sodalive.can.use.UseCanCalculateRepository
import kr.co.vividnext.sodalive.can.use.UseCanCalculateStatus
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.live.room.LiveRoomRepository
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.MemberRepository
import kr.co.vividnext.sodalive.member.MemberRole
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
class NewRouletteService(
    private val idGenerator: RedisIdGenerator,
    private val canPaymentService: CanPaymentService,

    private val canRepository: CanRepository,
    private val repository: NewRouletteRepository,
    private val oldRepository: RouletteRepository,
    private val roomRepository: LiveRoomRepository,
    private val memberRepository: MemberRepository,
    private val chargeRepository: ChargeRepository,
    private val useCanCalculateRepository: UseCanCalculateRepository
) {
    fun getAllRoulette(creatorId: Long, memberId: Long): List<GetNewRouletteResponse> {
        if (creatorId != memberId) throw SodaException("잘못된 요청입니다.")

        var rouletteList = repository.findByCreatorId(creatorId)
        if (rouletteList.isEmpty()) {
            val roulette = oldRepository.findByIdOrNull(creatorId)
            if (roulette != null) {
                repository.save(
                    NewRoulette(
                        id = idGenerator.generateId(SEQUENCE_NAME),
                        creatorId = creatorId,
                        can = roulette.can,
                        isActive = false,
                        items = roulette.items
                    )
                )

                rouletteList = repository.findByCreatorId(creatorId)
                oldRepository.deleteById(roulette.creatorId)
            }
        }

        return rouletteList.sortedBy { it.id }
            .asSequence()
            .map {
                GetNewRouletteResponse(
                    it.id,
                    it.can,
                    it.isActive,
                    it.items
                )
            }
            .toList()
    }

    fun createRoulette(memberId: Long, request: CreateNewRouletteRequest): Boolean {
        rouletteValidate(can = request.can, items = request.items)

        if (request.isActive) {
            val rouletteList = repository.findByCreatorId(creatorId = memberId)
            rouletteList.forEach {
                it.isActive = false
                repository.save(it)
            }
        }

        val roulette = NewRoulette(
            id = idGenerator.generateId(SEQUENCE_NAME),
            creatorId = memberId,
            can = request.can,
            isActive = request.isActive,
            items = request.items
        )

        repository.save(roulette)
        return request.isActive
    }

    fun updateRoulette(memberId: Long, request: UpdateNewRouletteRequest): Boolean {
        rouletteValidate(can = request.can, items = request.items)

        val rouletteList = repository.findByCreatorId(creatorId = memberId)

        if (rouletteList.isEmpty()) {
            throw SodaException("잘못된 요청입니다.")
        }

        var isActive = false
        rouletteList.forEach {
            if (request.isActive || it.isActive) {
                isActive = true
            }

            if (it.id == request.id) {
                it.can = request.can
                it.items = request.items
                it.isActive = request.isActive
                repository.save(it)
            } else if (request.isActive) {
                it.isActive = false
                repository.save(it)
            }
        }

        return isActive
    }

    fun getRoulette(creatorId: Long, memberId: Long): GetRouletteResponse {
        val rouletteList = repository.findByCreatorId(creatorId = creatorId)

        if (rouletteList.isEmpty()) {
            throw SodaException("룰렛을 사용할 수 없습니다.")
        }

        var activeRoulette: NewRoulette? = null
        for (roulette in rouletteList) {
            if (roulette.isActive) {
                activeRoulette = roulette
                break
            }
        }

        if (activeRoulette == null || activeRoulette.items.isEmpty()) {
            throw SodaException("룰렛을 사용할 수 없습니다.")
        }

        return GetRouletteResponse(
            can = activeRoulette.can,
            isActive = activeRoulette.isActive,
            items = activeRoulette.items
        )
    }

    @Transactional
    fun spinRoulette(request: SpinRouletteRequest, memberId: Long): GetRouletteResponse {
        // STEP 1 - 라이브 정보 가져오기
        val room = roomRepository.findByIdOrNull(request.roomId)
            ?: throw SodaException("해당하는 라이브가 없습니다.")

        val host = room.member ?: throw SodaException("잘못된 요청입니다.")

        if (host.role != MemberRole.CREATOR) {
            throw SodaException("비비드넥스트와 계약한\n크리에이터의 룰렛만 사용하실 수 있습니다.")
        }

        // STEP 2 - 룰렛 데이터 가져오기
        val rouletteList = repository.findByCreatorId(creatorId = host.id!!)

        if (rouletteList.isEmpty()) {
            throw SodaException("룰렛을 사용할 수 없습니다.")
        }

        var activeRoulette: NewRoulette? = null
        for (roulette in rouletteList) {
            if (roulette.isActive) {
                activeRoulette = roulette
                break
            }
        }

        if (activeRoulette == null || activeRoulette.items.isEmpty()) {
            throw SodaException("룰렛을 사용할 수 없습니다.")
        }

        // STEP 3 - 캔 사용
        canPaymentService.spendCan(
            memberId = memberId,
            needCan = activeRoulette.can,
            canUsage = CanUsage.SPIN_ROULETTE,
            liveRoom = room,
            container = request.container
        )

        return GetRouletteResponse(
            can = activeRoulette.can,
            isActive = activeRoulette.isActive,
            items = activeRoulette.items
        )
    }

    @Transactional
    fun refundDonation(roomId: Long, member: Member) {
        val donator = memberRepository.findByIdOrNull(member.id)
            ?: throw SodaException("룰렛 돌리기에 실패한 캔이 환불되지 않았습니다\n고객센터로 문의해주세요.")

        val useCan = canRepository.getCanUsedForLiveRoomNotRefund(
            memberId = member.id!!,
            roomId = roomId,
            canUsage = CanUsage.SPIN_ROULETTE
        ) ?: throw SodaException("룰렛 돌리기에 실패한 캔이 환불되지 않았습니다\n고객센터로 문의해주세요.")
        useCan.isRefund = true

        val useCanCalculates = useCanCalculateRepository.findByUseCanIdAndStatus(useCan.id!!)
        useCanCalculates.forEach {
            it.status = UseCanCalculateStatus.REFUND
            val charge = Charge(0, it.can, status = ChargeStatus.REFUND_CHARGE)
            charge.title = "${it.can} 캔"
            charge.useCan = useCan

            when (it.paymentGateway) {
                PaymentGateway.GOOGLE_IAP -> donator.googleRewardCan += charge.rewardCan
                PaymentGateway.APPLE_IAP -> donator.appleRewardCan += charge.rewardCan
                else -> donator.pgRewardCan += charge.rewardCan
            }
            charge.member = donator

            val payment = Payment(
                status = PaymentStatus.COMPLETE,
                paymentGateway = it.paymentGateway
            )
            payment.method = "룰렛 환불"
            charge.payment = payment

            chargeRepository.save(charge)
        }
    }

    private fun rouletteValidate(can: Int, items: List<RouletteItem>) {
        if (can < 5) {
            throw SodaException("룰렛 금액은 최소 5캔 입니다.")
        }

        if (items.size < 2 || items.size > 10) {
            throw SodaException("룰렛 옵션은 최소 2개, 최대 10개까지 설정할 수 있습니다.")
        }
    }

    companion object {
        const val SEQUENCE_NAME = "newRoulette:sequence"
    }
}
